@@ -81,6 +81,7 @@ def get_help():
             ["",     "",                        "TECHFORM",            "Check if cookie value format reveals technology"],
             ["",     "",                        "ACCVAL",              "Check if server accepts arbitrary cookie values"],
             ["",     "",                        "ACCURL",              "Check if server sets cookie from URL parameter"],
+            ["",     "",                        "FROMURL",             ""],
             ["",     "",                        "FPD",                 "Check if empty or invalid value triggers FPD"],
             ["", "", "", ""],
             ["-c", "--cookie-name",             "<cookie-name>",       "Parse only specific <cookie-name>"],
@@ -99,7 +100,7 @@ def get_help():
 
 
 def parse_args():
-    available_tests = ["IDENT", "SECURE", "HTTPONLY", "SAMESITE", "PREFIX", "DOMAIN", "TECHNAME", "TECHFORM", "ACCVAL", "ACCURL", "FPD"]
+    available_tests = ["IDENT", "SECURE", "HTTPONLY", "SAMESITE", "PREFIX", "DOMAIN", "TECHNAME", "TECHFORM", "ACCVAL", "ACCURL", "FROMURL", "FPD"]
     parser = argparse.ArgumentParser(add_help="False")
     parser.add_argument("-u",      "--url",               type=str, required=True)
     parser.add_argument("-ts",     "--tests",             type=lambda s: s.upper(), nargs="+", default=available_tests)
